/*

	Written by Mat Hopwood of Pocketeers Limited - www.drmop.com

	You are free to use this code for any purpose whatsoever, I do not provide any warranty

*/

// Marmalade headers
#include "s3e.h"
#include "Iw2D.h"

// LUA headers
extern "C"
{
   #include "lua.h"
   #include "lauxlib.h"
   #include "lualib.h"
}


void CallLuaString(const char* string)
{
	s3eDebugOutputString("==== Calling a LUA string");

	// Open LUA
	lua_State *lua = lua_open();

	// Add LUA libraries
	luaL_openlibs(lua);										// See http://www.lua.org/manual/5.1/manual.html#luaL_openlibs

	// Pass the string to lua to execute
	if (luaL_loadbuffer(lua, string, strlen(string), "line") == 0)	// See http://www.lua.org/manual/5.1/manual.html#luaL_loadbuffer
	{
		if (lua_pcall(lua, 0, 0, 0) != 0)
		{
			// Output the error
			s3eDebugOutputString(lua_tostring(lua, -1));

			// Pop error message off the stack
			lua_pop(lua, 1);								// see http://www.lua.org/manual/5.1/manual.html#lua_pop
		}
	}

	// Shut down the lua state
	lua_close(lua);											// See http://www.lua.org/manual/5.1/manual.html#lua_close
 
}

void CallLuaFile(const char* lua_filename)
{
	s3eDebugOutputString("==== Calling a LUA file");

	// Open LUA
	lua_State *lua = lua_open();

	// Add LUA libraries
	luaL_openlibs(lua);										// See http://www.lua.org/manual/5.1/manual.html#luaL_openlibs

	// Load our test LUA file
	if (luaL_loadfile(lua, lua_filename) == 0)				// See http://www.lua.org/manual/5.1/manual.html#lua_load
	{
		if (lua_pcall(lua, 0, 0, 0) != 0)					// See http://www.lua.org/manual/5.1/manual.html#lua_pcall
		{
			// Output the error
			s3eDebugOutputString(lua_tostring(lua, -1));

			// Pop error message off the stack
			lua_pop(lua, 1);								// see http://www.lua.org/manual/5.1/manual.html#lua_pop
		}
	}

	// Shut down the lua state
	lua_close(lua);											// See http://www.lua.org/manual/5.1/manual.html#lua_close
}

void CallLuaFunctionInFile(const char* lua_filename, const char* function_name, double arg0, double arg1)
{
	s3eDebugOutputString("==== Calling a LUA function from C");

	// Open LUA
	lua_State *lua = lua_open();

	// Add LUA libraries
	luaL_openlibs(lua);										// See http://www.lua.org/manual/5.1/manual.html#luaL_openlibs

	// Load our test LUA file
	if (luaL_loadfile(lua, lua_filename) == 0)				// See http://www.lua.org/manual/5.1/manual.html#lua_load
	{
		// Init the loaded lua file
		if (lua_pcall(lua, 0, 0, 0) == 0)					// See http://www.lua.org/manual/5.1/manual.html#lua_pcall
		{
			// Push the name of the function that we want to call onto the stack
			// NB: LUA uses a stack to pass information back and forth between LUA and C (see http://www.lua.org/pil/24.2.html)
			lua_getglobal(lua, function_name);				// Push function name that we want to call ontothe stack (see http://www.lua.org/manual/5.1/manual.html#lua_getglobal)
			lua_pushnumber(lua, arg0);						// Push function call argument 1
			lua_pushnumber(lua, arg1);						// Push function call argument 2

			if (lua_pcall(lua, 2, 1, 0) != 0)				// See http://www.lua.org/manual/5.1/manual.html#lua_pcall
				s3eDebugOutputString(lua_tostring(lua, -1));
			else
			{
				if (!lua_isnumber(lua, -1))					// http://www.lua.org/manual/5.1/manual.html#lua_isnumber
					s3eDebugOutputString("Function muust return a value");
				else
				{
					// Get th result returned from the LUA function
					double result = lua_tonumber(lua, -1);
				}
			}
			lua_pop(lua, 1);								// see http://www.lua.org/manual/5.1/manual.html#lua_pop
		}
	}

	// Shut down the lua state
	lua_close(lua);											// See http://www.lua.org/manual/5.1/manual.html#lua_close
}

static int test_function(lua_State *lua)
{
	double d = 0;
	// Get the argument that was passed from lua
	if (lua_isnumber(lua, 1))
		d = lua_tonumber(lua, 1);
	else
		s3eDebugOutputString("test_function can only accept a number");

	// Square it
	d = d * d;
	
	// Push the result back onto the stack as a return value
	lua_pushnumber(lua, d);

	// Return the number of result arguments that were passed back to lua
	return 1;
}

void CallCFunctionFromLua()
{
	s3eDebugOutputString("==== Calling a C function from LUA");

	// Open LUA
	lua_State *lua = lua_open();

	// Add LUA libraries
	luaL_openlibs(lua);										// See http://www.lua.org/manual/5.1/manual.html#luaL_openlibs

	// Push the test function
	lua_pushcfunction(lua, test_function);					// See http://www.lua.org/manual/5.1/manual.html#lua_pushcfunction

	// Set the global test_function to the pushed C function
	lua_setglobal(lua, "test_function");					// See http://www.lua.org/manual/5.1/manual.html#lua_setglobal

	// Load our test LUA file
	if (luaL_loadfile(lua, "lua_test3.lua") == 0)			// See http://www.lua.org/manual/5.1/manual.html#lua_load
	{
		if (lua_pcall(lua, 0, 0, 0) != 0)					// See http://www.lua.org/manual/5.1/manual.html#lua_pcall
		{
			// Output the error
			s3eDebugOutputString(lua_tostring(lua, -1));

			// Pop error message off the stack
			lua_pop(lua, 1);								// see http://www.lua.org/manual/5.1/manual.html#lua_pop
		}
	}

	// Shut down the lua state
	lua_close(lua);											// See http://www.lua.org/manual/5.1/manual.html#lua_close}
}

int main()
{
	// Init Marmalade 2D graphics
    Iw2DInit();

	// LUA tests
	CallLuaString("print(\"Hello World\");\n");
	CallLuaFile("lua_test1.lua");
	CallLuaFunctionInFile("lua_test2.lua", "AddStuff", 10, 20);
	CallCFunctionFromLua();

   	// Main Game Loop
	while (!s3eDeviceCheckQuitRequest())
	{
		// Update keyboard system
		s3eKeyboardUpdate();
		if (s3eKeyboardGetState(s3eKeyAbsBSK) & S3E_KEY_STATE_DOWN)	// Back key is used to exit on some platforms
			break;

		// Update pointer system
		s3ePointerUpdate();

		// Clear the screen
		Iw2DSurfaceClear(0);

		// Show the surface
		Iw2DSurfaceShow();

		// Yield to the opearting system
		s3eDeviceYield(0);
	}

	// Shut down Marmalade 2D graphics
	Iw2DTerminate();

	return 0;
}

